package br.com.impl.knowledge;

import java.util.Collection;
import java.util.Map;

import org.drools.KnowledgeBase;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.drools.runtime.process.WorkItemHandler;

import br.com.api.knowledge.EasyPieKnowledgeSessionInterface;

public class EasyPieKnowledgeSessionImpl implements EasyPieKnowledgeSessionInterface {

	private StatefulKnowledgeSession	ksession;

	public EasyPieKnowledgeSessionImpl() {

	}

	public EasyPieKnowledgeSessionImpl(final StatefulKnowledgeSession ksession) {

		this.ksession = ksession;
	}

	public StatefulKnowledgeSession getKsession() {
		
		return this.ksession;
	}

	public Collection<ProcessInstance> getProcessIntances() {

		return this.ksession.getProcessInstances();
	}

	/**
	 * Instancia um novo StatefulKnowledgeSession baseado na KnowledgeBase fornecida.
	 * Associa essa objeto à variavel de referência 'ksession'.
	 * 
	 * @params kbase - KnowledgeBase usado para instanciar um novo StatefulKnowledgeSession
	 */
	public StatefulKnowledgeSession newStatefulKnowledgeSession(final KnowledgeBase kbase) {

		this.ksession = kbase.newStatefulKnowledgeSession();
		return this.ksession;
	}
	
	public StatefulKnowledgeSession newStatefulKnowledgeSession(KnowledgeBase kbase, Environment env) {
		
		this.ksession = JPAKnowledgeService.newStatefulKnowledgeSession(kbase, null, env);
		
		return this.ksession;
	}

	/**
	 * Regitra um novo WorkItemHandler.
	 * 
	 * @param name
	 *            - Name da task que se referencia.
	 * @param workItemHandlerImplementation
	 *            - Implementação da WorkItemHandler que será registrado.
	 */
	public void registerWorkItemHandler(final String name, final WorkItemHandler workItemHandlerImplementation) {

		this.ksession.getWorkItemManager().registerWorkItemHandler(name, workItemHandlerImplementation);
	}

	public void setKsession(final StatefulKnowledgeSession ksession) {

		this.ksession = ksession;
	}

	// getters and setters

	/**
	 * Inicia o processo a partir do caminho de processo fornecido.
	 * 
	 * @param processPath
	 *            - Caminho do processo desejado.
	 */
	public ProcessInstance startProcess(final String processPath) {

		return this.ksession.startProcess(processPath);
	}

	/**
	 * Inicia o processo a partir do caminho de processo fornecido.
	 * 
	 * @param processPath
	 *            - Caminho do processo desejado.
	 * @param params
	 *            - Parâmetros que serão utilizados no processo.
	 */
	public ProcessInstance startProcess(final String processPath, final Map<String, Object> params) {

		return this.ksession.startProcess(processPath, params);
	}

}
