package br.com.impl.task;

import java.util.List;

import org.drools.runtime.StatefulKnowledgeSession;
import org.jbpm.process.workitem.wsht.AsyncGenericHTWorkItemHandler;
import org.jbpm.process.workitem.wsht.AsyncHornetQHTWorkItemHandler;
import org.jbpm.task.Content;
import org.jbpm.task.Status;
import org.jbpm.task.Task;
import org.jbpm.task.query.TaskSummary;
import org.jbpm.task.service.ContentData;
import org.jbpm.task.service.FaultData;
import org.jbpm.task.service.TaskClient;
import org.jbpm.task.service.responsehandlers.AbstractBlockingResponseHandler;
import org.jbpm.task.service.responsehandlers.BlockingGetContentResponseHandler;
import org.jbpm.task.service.responsehandlers.BlockingGetTaskResponseHandler;
import org.jbpm.task.service.responsehandlers.BlockingTaskOperationResponseHandler;
import org.jbpm.task.service.responsehandlers.BlockingTaskSummaryResponseHandler;

import br.com.api.task.EasyPieLocalTaskServiceInterface;

public class EasyPieLocalTaskServiceImpl implements EasyPieLocalTaskServiceInterface {

	private TaskClient client;

	private static final int WAIT_TIME = 1000;

	public void assignTaskClient(final TaskClient taskClient){

		client = taskClient;
	}

	public AsyncHornetQHTWorkItemHandler createTaskHandler(final StatefulKnowledgeSession ksession, TaskClient	taskClient) {

		client = taskClient;

		AsyncHornetQHTWorkItemHandler htHandler = new AsyncHornetQHTWorkItemHandler(taskClient, ksession, false);

		return htHandler;
	}

	private void waitToComplete(AbstractBlockingResponseHandler responseHandler) {

		int count = 0;

		while(!responseHandler.isDone() && count < 10){

			responseHandler.waitTillDone(WAIT_TIME);

			count++;
		}

	}

	// métodos de LocalTaskService

	
	public void claim(final long taskId, final String userId) {

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.claim(taskId, userId, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public void claimNextAvailable(final String userId, final String language) {

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.claimNextAvailable(userId, language, responseHandler);

		waitToComplete(responseHandler);

	}

	
	public void complete(final long taskId, final String userId, final ContentData outputData) {

		System.out.println("COMPLETE");

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.complete(taskId, userId, outputData, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public void completeWithResults(final long taskId, final String userId, final Object results) {

		System.out.println("COMPLETE WITH RESULTS");

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.completeWithResults(taskId, userId, results, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public void delegate(final long taskId, final String userId, final String targetUserId) {

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.delegate(taskId, userId, targetUserId, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public void exit(final long taskId, final String userId) {

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.exit(taskId, userId, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public void fail(final long taskId, final String userId, final FaultData faultData) {

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.fail(taskId, userId, faultData, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public void forward(final long taskId, final String userId, final String targetEntityId) {

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.forward(taskId, userId, targetEntityId, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public List<TaskSummary> getTasksAssignedAsPotentialOwner(final String userId, final String language) {

		System.out.println("getTasksAssignedAsPotentialOwner");

		BlockingTaskSummaryResponseHandler responseHandler = new BlockingTaskSummaryResponseHandler();

		client.getTasksAssignedAsPotentialOwner(userId, language, responseHandler);

		waitToComplete(responseHandler);

		List<TaskSummary> tasks = responseHandler.getResults();

		return tasks;
	}

	
	public List<TaskSummary> getTasksAssignedAsPotentialOwnerByStatus(final String userId, final List<Status> status, final String language) {

		System.out.println("getTasksAssignedAsPotentialOwnerByStatus");

		BlockingTaskSummaryResponseHandler responseHandler = new BlockingTaskSummaryResponseHandler();

		client.getTasksAssignedAsPotentialOwnerByStatus(userId, status, language, responseHandler);

		waitToComplete(responseHandler);

		List<TaskSummary> tasks = responseHandler.getResults();

		return tasks;
	}

	
	public List<TaskSummary> getTasksByStatusByProcessId(final long processInstanceId, final List<Status> status, final String language) {

		BlockingTaskSummaryResponseHandler responseHandler = new BlockingTaskSummaryResponseHandler();

		client.getTasksByStatusByProcessId(processInstanceId, status, language, responseHandler);

		waitToComplete(responseHandler);

		List<TaskSummary> tasks = responseHandler.getResults();

		return tasks;
	}

	
	public List<TaskSummary> getTasksByStatusByProcessIdByTaskName(final long processInstanceId, final List<Status> status, final String taskName, final String language) {

		BlockingTaskSummaryResponseHandler responseHandler = new BlockingTaskSummaryResponseHandler();

		client.getTasksByStatusByProcessIdByTaskName(processInstanceId, status, taskName, language, responseHandler);

		waitToComplete(responseHandler);

		List<TaskSummary> tasks = responseHandler.getResults();

		return tasks;
	}

	
	public List<TaskSummary> getTasksOwned(final String userId, final List<Status> status, final String language) {

		BlockingTaskSummaryResponseHandler responseHandler = new BlockingTaskSummaryResponseHandler();

		client.getTasksOwned(userId, status, language, responseHandler);

		waitToComplete(responseHandler);

		return responseHandler.getResults();
	}

	
	public List<TaskSummary> getTasksOwned(final String userId, final String language) {

		BlockingTaskSummaryResponseHandler responseHandler = new BlockingTaskSummaryResponseHandler();

		client.getTasksOwned(userId, language, responseHandler);

		waitToComplete(responseHandler);

		return responseHandler.getResults();
	}

	
	public void release(final long taskId, final String userId) {

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.release(taskId, userId, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public void remove(final long taskId, final String userId) {

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.remove(taskId, userId, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public void resume(final long taskId, final String userId) {

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.resume(taskId, userId, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public void skip(final long taskId, final String userId) {

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.skip(taskId, userId, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public void start(final long taskId, final String userId) {

		System.out.println("START TASK");

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.start(taskId, userId, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public void stop(final long taskId, final String userId) {

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.stop(taskId, userId, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public void suspend(final long taskId, final String userId) {

		BlockingTaskOperationResponseHandler responseHandler = new BlockingTaskOperationResponseHandler();

		client.suspend(taskId, userId, responseHandler);

		waitToComplete(responseHandler);
	}

	
	public Content getContent(long contentId) {

		BlockingGetContentResponseHandler responseHandler = new BlockingGetContentResponseHandler();

		client.getContent(contentId, responseHandler);

		waitToComplete(responseHandler);

		return responseHandler.getContent();
	}

	
	public Task getTask(long taskId) {

		BlockingGetTaskResponseHandler responseHandler = new BlockingGetTaskResponseHandler();

		client.getTask(taskId, responseHandler);

		waitToComplete(responseHandler);

		return responseHandler.getTask();
	}

	
	public Task getTaskByWorkItemId(long workItemId) {

		BlockingGetTaskResponseHandler responseHandler = new BlockingGetTaskResponseHandler();

		client.getTaskByWorkItemId(workItemId, responseHandler);

		waitToComplete(responseHandler);

		return responseHandler.getTask();
	}

}
